@@ -22,18 +22,24 @@
  *
  */
 
+#define _GNU_SOURCE
+#define _LARGEFILE64_SOURCE
 #include <features.h>
+#undef __OPTIMIZE__
+/* We absolutely do _NOT_ want interfaces silently
+ *  *  * renamed under us or very bad things will happen... */
+#ifdef __USE_FILE_OFFSET64
+# undef __USE_FILE_OFFSET64
+#endif
+
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/syscall.h>
+#include <endian.h>
+
 
 //#define __NR_exit             1
-#ifdef L__exit
-/* Do not include unistd.h, so gcc doesn't whine about 
- * _exit returning.  It really doesn't return... */
-#define __NR__exit __NR_exit
-_syscall1(void, _exit, int, status);
-#endif
+//See _exit.c
 
 //#define __NR_fork             2
 #ifdef L___libc_fork
@@ -65,6 +71,9 @@ weak_alias(__libc_read, read)
 #define __NR___libc_write __NR_write
 _syscall3(ssize_t, __libc_write, int, fd, const __ptr_t, buf, size_t, count);
 weak_alias(__libc_write, write)
+/* Stupid libgcc.a from gcc 2.95.x uses __write in pure.o
+ * which is a blatent GNU libc-ism... */
+weak_alias (__libc_write, __write)
 #endif
 
 //#define __NR_open             5
@@ -90,7 +99,15 @@ weak_alias(__libc_close, close)
 //#define __NR_creat            8
 #ifdef L_creat
 #include <fcntl.h>
+#ifdef __NR_creat
 _syscall2(int, creat, const char *, file, mode_t, mode);
+#else
+extern int __libc_open (const char *file, int flags, mode_t mode);
+int creat (const char *file, mode_t mode)
+{
+	  return __libc_open (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
+}
+#endif
 #endif
 
 //#define __NR_link             9
@@ -121,25 +138,34 @@ _syscall1(int, chdir, const char *, path);
 //#define __NR_time             13
 #ifdef L_time
 #include <time.h>
+#include <sys/time.h>
+#ifdef __NR_time
 _syscall1(time_t, time, time_t *, t);
+#else
+time_t time (time_t *t)
+{ 
+	time_t result;
+	struct timeval tv;
+	if (gettimeofday (&tv, (struct timezone *) NULL)) {
+		result = (time_t) -1;
+	} else { result = (time_t) tv.tv_sec; }
+	if (t != NULL) { *t = result; }
+	return result;
+}
+#endif
 #endif
 
 //#define __NR_mknod            14
-#ifdef L_mknod
-#include <unistd.h>
-extern int mknod(const char *pathname, mode_t mode, dev_t dev);
-_syscall3(int, mknod, const char *, pathname, mode_t, mode, dev_t, dev);
-
-int __xmknod (int version, const char * path, mode_t mode, dev_t *dev)
-{
-	switch(version)
-	{
-		case 1:
-			return mknod (path, mode, *dev);
-		default:
-			__set_errno(EINVAL);
-			return -1;
-	}
+#ifdef L___syscall_mknod
+#define __NR___syscall_mknod __NR_mknod
+#include <sys/stat.h>
+_syscall3(int, __syscall_mknod, const char *, path, __kernel_mode_t, mode, __kernel_dev_t, dev);
+int mknod(const char *path, mode_t mode, dev_t dev)
+{ 
+	__kernel_dev_t k_dev;
+	/* We must convert the dev_t value to a __kernel_dev_t */
+	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
+	return __syscall_mknod(path, mode, k_dev);
 }
 #endif
 
@@ -178,6 +204,9 @@ weak_alias(__libc_lseek, lseek)
 //#define __NR_getpid           20
 #ifdef L___libc_getpid
 #include <unistd.h>
+#if defined (__alpha__)
+#define __NR_getpid     __NR_getxpid
+#endif
 #define __NR___libc_getpid __NR_getpid
 _syscall0(pid_t, __libc_getpid);
 weak_alias(__libc_getpid, getpid)
@@ -199,31 +228,33 @@ _syscall1(int, umount, const char *, specialfile);
 #endif
 
 //#define __NR_setuid           23
-#ifdef L___setuid
-#define __NR___setuid __NR_setuid
-#include <unistd.h>
-static inline 
-_syscall1(int, __setuid, uid_t, uid);
-int setuid(uid_t uid)
-{
-	if (uid == (uid_t) ~0) {
-		__set_errno (EINVAL);
-		return -1;
-	}
-	return(__setuid(uid));
-}
-#endif
+// See setuid.c
 
 //#define __NR_getuid           24
 #ifdef L_getuid
 #include <unistd.h>
+#if defined (__alpha__)
+#define __NR_getuid     __NR_getxuid
+#endif
 _syscall0(uid_t, getuid);
 #endif
 
 //#define __NR_stime            25
 #ifdef L_stime
 #include <time.h>
+#include <sys/time.h>
+#ifdef _NR_stime
 _syscall1(int, stime, const time_t *, t);
+#else
+int stime(const time_t *when)
+{ 
+	struct timeval tv;
+	if (when == NULL) { __set_errno (EINVAL); return -1; }
+	tv.tv_sec = *when;
+	tv.tv_usec = 0;
+	return settimeofday (&tv, (struct timezone *) 0);
+}
+#endif
 #endif
 
 //#define __NR_ptrace           26
@@ -238,7 +269,24 @@ _syscall4(long, __ptrace, enum __ptrace_request, request, pid_t, pid,
 //#define __NR_alarm            27
 #ifdef L_alarm
 #include <unistd.h>
+#ifdef __NR_alarm
 _syscall1(unsigned int, alarm, unsigned int, seconds);
+#else
+#include <sys/time.h>
+unsigned int alarm (unsigned int seconds)
+{
+	struct itimerval old, new;
+	unsigned int retval;
+	new.it_value.tv_usec = 0;
+	new.it_interval.tv_sec = 0;
+	new.it_interval.tv_usec = 0;
+	new.it_value.tv_sec = (long int) seconds;
+	if (setitimer (ITIMER_REAL, &new, &old) < 0) { return 0; }
+	retval = old.it_value.tv_sec;
+	if (old.it_value.tv_usec) { ++retval; }
+	return retval;
+}
+#endif
 #endif
 
 //#define __NR_oldfstat         28
@@ -246,15 +294,64 @@ _syscall1(unsigned int, alarm, unsigned int, seconds);
 //#define __NR_pause            29
 #ifdef L___libc_pause
 #include <unistd.h>
+#ifdef __NR_pause
 #define __NR___libc_pause __NR_pause
 _syscall0(int, __libc_pause);
 weak_alias(__libc_pause, pause)
+#else
+#include <signal.h>
+int __libc_pause (void)
+{
+	return(__sigpause(sigblock(0), 0));
+}
+weak_alias(__libc_pause, pause)
+#endif
 #endif
 
 //#define __NR_utime            30
 #ifdef L_utime
 #include <utime.h>
-_syscall2(int, utime, const char *, filename, const struct utimbuf *, buf);
+#ifdef __NR_utime
+_syscall2(int, utime, const char *, file, const struct utimbuf *, times);
+#else
+#include <stdlib.h>
+#include <sys/time.h>
+int utime(const char *file, const struct utimbuf *times)
+{
+	struct timeval timevals[2];
+	if (times != NULL) {
+		timevals[0].tv_usec = 0L;
+		timevals[1].tv_usec = 0L;
+		timevals[0].tv_sec = (long int) times->actime;
+		timevals[1].tv_sec = (long int) times->modtime;
+	} else {
+		if (gettimeofday (&timevals[0], NULL) < 0) { return -1; }
+		timevals[1] = timevals[0];
+	}
+	return utimes(file, timevals);
+}
+#endif
+#endif
+
+//#define __NR_utimed
+#ifdef L_utimes
+#include <utime.h>
+#ifdef __NR_utimes
+_syscall2(int, utimes, const char *, file, const struct timeval *, tvp);
+#else
+#include <stdlib.h>
+#include <sys/time.h>
+int utimes (const char *file, const struct timeval tvp[2])
+{
+	struct utimbuf buf, *times;
+	if (tvp) {
+		times = &buf;
+		times->actime = tvp[0].tv_sec;
+		times->modtime = tvp[1].tv_sec;
+	} else { times = NULL; }
+	return utime(file, times);
+}
+#endif
 #endif
 
 //#define __NR_stty             31
@@ -286,16 +383,30 @@ _syscall2(int, access, const char *, pathname, int, mode);
 //#define __NR_nice             34
 #ifdef L_nice
 #include <unistd.h>
+#ifdef __NR_nice
 _syscall1(int, nice, int, inc);
+#else
+#include <sys/resource.h>
+int nice (int incr)
+{
+	int save, prio, result;
+	save = errno;
+	__set_errno (0);
+	prio = getpriority (PRIO_PROCESS, 0);
+	if (prio == -1) {
+		if (errno != 0) { return -1; } 
+		else { __set_errno (save); }
+	}
+	result = setpriority (PRIO_PROCESS, 0, prio + incr);
+	if (result != -1) { return prio + incr; } else { return -1; }
+}
+#endif
 #endif
 
 //#define __NR_ftime            35
 
 //#define __NR_sync             36
-#ifdef L_sync
-#include <unistd.h>
-_syscall0(void, sync);
-#endif
+//See sync.c
 
 //#define __NR_kill             37
 #ifdef L_kill
@@ -353,6 +464,9 @@ _syscall1(int, setgid, gid_t, gid);
 //#define __NR_getgid           47
 #ifdef L_getgid
 #include <unistd.h>
+#if defined (__alpha__)
+#define __NR_getgid     __NR_getxgid
+#endif
 _syscall0(gid_t, getgid);
 #endif
 
@@ -360,8 +474,8 @@ _syscall0(gid_t, getgid);
 
 //#define __NR_geteuid          49
 #ifdef	L_geteuid
+#include <unistd.h>
 #	ifdef	__NR_geteuid
-#	include <unistd.h>
 	_syscall0(uid_t, geteuid);
 #	else
 	uid_t geteuid(void)
@@ -373,8 +487,8 @@ _syscall0(gid_t, getgid);
 
 //#define __NR_getegid          50
 #ifdef	L_getegid
+#include <unistd.h>
 #	ifdef	__NR_getegid
-#	include <unistd.h>
 	_syscall0(gid_t, getegid);
 #	else
 	gid_t getegid(void)
@@ -510,11 +624,11 @@ _syscall0(pid_t, setsid);
 
 //#define __NR_sigaction        67
 #ifndef __NR_rt_sigaction
-#define __NR___sigaction __NR_sigaction
-#ifdef L___sigaction
+#define __NR___syscall_sigaction __NR_sigaction
+#ifdef L___syscall_sigaction
 #include <signal.h>
 #undef sigaction
-_syscall3(int, __sigaction, int, signum, const struct sigaction *, act,
+_syscall3(int, __syscall_sigaction, int, signum, const struct sigaction *, act,
 		  struct sigaction *, oldact);
 #endif
 #endif
@@ -590,8 +704,8 @@ int setrlimit (__rlimit_resource_t resource, const struct rlimit *rlimits)
 #else /* We don't need to wrap setrlimit */
 #ifdef L_setrlimit
 #include <unistd.h>
-#include <sys/resource.h>
-_syscall2(int, setrlimit, int, resource, const struct rlimit *, rlim);
+struct rlimit;
+_syscall2(int, setrlimit, unsigned int, resource, const struct rlimit *, rlim);
 #endif
 #endif /* __NR_setrlimit */
 
@@ -656,6 +770,16 @@ _syscall2(int, setgroups, size_t, size, const gid_t *, list);
 #endif
 
 //#define __NR_select           82
+#ifdef L_select
+//Used as a fallback if _newselect isn't available...
+#ifndef __NR__newselect
+#include <unistd.h>
+extern int select(int n, fd_set *readfds, fd_set *writefds, 
+		fd_set *exceptfds, struct timeval *timeout);
+_syscall5(int, select, int, n, fd_set *, readfds, fd_set *, writefds,
+		fd_set *, exceptfds, struct timeval *, timeout);
+#endif
+#endif
 
 //#define __NR_symlink          83
 #ifdef L_symlink
@@ -754,9 +878,22 @@ _syscall3(int, fchown, int, fd, uid_t, owner, gid_t, group);
 #endif
 
 //#define __NR_getpriority      96
-#ifdef L_getpriority
+#ifdef L___syscall_getpriority
 #include <sys/resource.h>
-_syscall2(int, getpriority, __priority_which_t, which, id_t, who);
+#define __NR___syscall_getpriority __NR_getpriority
+_syscall2(int, __syscall_getpriority, __priority_which_t, which, id_t, who);
+/* The return value of __syscall_getpriority is biased by this value
+ * to avoid returning negative values.  */
+#define PZERO 20
+int getpriority (enum __priority_which which, id_t who)
+{
+	int res;
+
+	res = __syscall_getpriority(which, who);
+	if (res >= 0)
+		res = PZERO - res;
+	return res;
+}
 #endif
 
 //#define __NR_setpriority      97
@@ -793,8 +930,11 @@ _syscall2(int, fstatfs, int, fd, struct statfs *, buf);
 #endif
 
 //#define __NR_socketcall       102
-#ifdef L_socketcall
-_syscall2(int, socketcall, int, call, unsigned long *, args);
+#ifdef L___socketcall
+#ifdef __NR_socketcall
+#define __NR___socketcall __NR_socketcall
+_syscall2(int, __socketcall, int, call, unsigned long *, args);
+#endif
 #endif
 
 //#define __NR_syslog           103
@@ -826,86 +966,84 @@ _syscall2(int, getitimer, __itimer_which_t, which, struct itimerval *, value);
 #endif
 
 //#define __NR_stat             106
-#ifdef L___stat
+#ifdef L___syscall_stat
+#define __NR___syscall_stat __NR_stat
 #include <unistd.h>
-#include "statfix.h"
-#define __NR___stat	__NR_stat
-extern int __stat(const char *file_name, struct kernel_stat *buf);
-_syscall2(int, __stat, const char *, file_name, struct kernel_stat *, buf);
-
-int __xstat(int version, const char * file_name, struct libc_stat * cstat)
+#define _SYS_STAT_H
+#include <bits/stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_stat, const char *, file_name, struct kernel_stat *, buf);
+int stat(const char * file_name, struct stat * buf)
 {
-	struct kernel_stat kstat;
-	int result = __stat(file_name, &kstat);
-
-	if (result == 0) { 
-		statfix(cstat, &kstat);
+	int result;
+	struct kernel_stat kbuf;
+	result = __syscall_stat(file_name, &kbuf);
+	if (result == 0) {
+		__xstat_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int stat(const char *file_name, struct libc_stat *buf)
-{
-	return(__xstat(0, file_name, buf));
-}
+#if ! defined __NR_stat64 && defined __UCLIBC_HAS_LFS__
+weak_alias(stat, stat64);
+#endif
 #endif
 
 //#define __NR_lstat            107
-#ifdef L___lstat
+#ifdef L___syscall_lstat
+#define __NR___syscall_lstat __NR_lstat
 #include <unistd.h>
-#include "statfix.h"
-#define __NR___lstat	__NR_lstat
-extern int __lstat(const char *file_name, struct kernel_stat *buf);
-_syscall2(int, __lstat, const char *, file_name, struct kernel_stat *, buf);
-
-int __lxstat(int version, const char * file_name, struct libc_stat * cstat)
+#define _SYS_STAT_H
+#include <bits/stat.h>
+#include <bits/kernel_stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_lstat, const char *, file_name, struct kernel_stat *, buf);
+int lstat(const char * file_name, struct stat * buf)
 {
-	struct kernel_stat kstat;
-	int result = __lstat(file_name, &kstat);
-
-	if (result == 0) { 
-		statfix(cstat, &kstat);
+	int result;
+	struct kernel_stat kbuf;
+	result = __syscall_lstat(file_name, &kbuf);
+	if (result == 0) {
+		__xstat_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int lstat(const char *file_name, struct libc_stat *buf)
-{
-	return(__lxstat(0, file_name, buf));
-}
+#if ! defined __NR_lstat64 && defined __UCLIBC_HAS_LFS__
+weak_alias(lstat, lstat64);
+#endif
 #endif
 
 //#define __NR_fstat            108
-#ifdef L___fstat
+#ifdef L___syscall_fstat
+#define __NR___syscall_fstat __NR_fstat
 #include <unistd.h>
-#include "statfix.h"
-#define __NR___fstat	__NR_fstat
-extern int __fstat(int filedes, struct kernel_stat *buf);
-_syscall2(int, __fstat, int, filedes, struct kernel_stat *, buf);
-
-int __fxstat(int version, int fd, struct libc_stat * cstat)
+#define _SYS_STAT_H
+#include <bits/stat.h>
+#include <bits/kernel_stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_fstat, int, fd, struct kernel_stat *, buf);
+int fstat(int fd, struct stat * buf)
 {
-	struct kernel_stat kstat;
-	int result = __fstat(fd, &kstat);
-
-	if (result == 0) { 
-		statfix(cstat, &kstat);
+	int result;
+	struct kernel_stat kbuf;
+	result = __syscall_fstat(fd, &kbuf);
+	if (result == 0) {
+		__xstat_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int fstat(int filedes, struct libc_stat *buf)
-{
-	return(__fxstat(0, filedes, buf));
-}
+#if ! defined __NR_fstat64 && defined __UCLIBC_HAS_LFS__
+weak_alias(fstat, fstat64);
+#endif
 #endif
 
 //#define __NR_olduname         109
 
 //#define __NR_iopl             110
 #ifdef L_iopl
+/* For arm there is a totally different implementation */
+#if !defined(__arm__)
 /* Tuns out the m68k unistd.h kernel header is broken */
-#	if defined __UCLIBC_HAS_MMU__ && defined __NR_iopl && ! defined(__mc68000__)
+#	if defined __UCLIBC_HAS_MMU__ && defined __NR_iopl && ( !defined(__mc68000__))
 		_syscall1(int, iopl, int, level);
 #	else
 		int iopl(int level)
@@ -914,6 +1052,7 @@ int fstat(int filedes, struct libc_stat *buf)
 			return -1;
 		}
 #	endif
+# endif
 #endif
 
 //#define __NR_vhangup          111
@@ -946,9 +1085,11 @@ _syscall1(int, sysinfo, struct sysinfo *, info);
 
 //#define __NR_ipc              117
 #ifdef L___ipc
+#ifdef __NR_ipc
 #define __NR___ipc __NR_ipc
 _syscall5(int, __ipc, unsigned int, call, int, first, int, second, int, third, void *, ptr);
 #endif
+#endif
 
 //#define __NR_fsync            118
 #ifdef L___libc_fsync
@@ -977,6 +1118,11 @@ _syscall1(int, uname, struct utsname *, buf);
 #endif
 
 //#define __NR_modify_ldt       123
+#ifdef __NR_modify_ldt
+#ifdef L_modify_ldt
+_syscall3(int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount);
+#endif
+#endif
 
 //#define __NR_adjtimex         124
 #ifdef L_adjtimex
@@ -1080,38 +1226,15 @@ _syscall1(int, setfsgid, gid_t, gid);
 #endif
 
 //#define __NR__llseek          140
-#ifdef L__llseek
-#ifdef __UCLIBC_HAVE_LFS__
-extern int _llseek(int fd, __off_t offset_hi, __off_t offset_lo, 
-		__loff_t *result, int whence);
-
-_syscall5(int, _llseek, int, fd, __off_t, offset_hi, __off_t, offset_lo, 
-		__loff_t *, result, int, whence);
-
-__loff_t __libc_lseek64(int fd, __loff_t offset, int whence)
-{
-	int ret;
-	__loff_t result;
-
-	ret = _llseek(fd, (__off_t) (offset >> 32),
-				  (__off_t) (offset & 0xffffffff), &result, whence);
-
-	return ret ? (__loff_t) ret : result;
-}
-weak_alias(__libc_lseek64, llseek);
-weak_alias(__libc_lseek64, lseek64);
-#endif
-#endif
+//See llseek.c
 
 //#define __NR_getdents         141
-#ifdef L_getdents
-#include <unistd.h>
-#include <dirent.h>
-_syscall3(int, getdents, int, fd, char *, dirp, size_t, count);
-#endif
+// See getdents.c
 
 //#define __NR__newselect       142
 #ifdef L__newselect
+//Used in preference to select when available...
+#ifdef __NR__newselect
 #include <unistd.h>
 extern int _newselect(int n, fd_set *readfds, fd_set *writefds,
 					  fd_set *exceptfds, struct timeval *timeout);
@@ -1119,6 +1242,7 @@ _syscall5(int, _newselect, int, n, fd_set *, readfds, fd_set *, writefds,
 		fd_set *, exceptfds, struct timeval *, timeout);
 weak_alias(_newselect, select);
 #endif
+#endif
 
 //#define __NR_flock            143
 #ifdef L_flock
@@ -1164,6 +1288,27 @@ _syscall1(int, fdatasync, int, fd);
 #endif
 
 //#define __NR__sysctl          149
+#ifdef __NR__sysctl
+#ifdef L__sysctl
+#include <linux/sysctl.h>
+_syscall1(int, _sysctl, struct __sysctl_args *, args);
+int sysctl(int *name, int nlen, void *oldval, size_t *oldlenp,
+			void *newval, size_t newlen)
+{
+	struct __sysctl_args args =
+	{
+		name: name,
+		nlen: nlen,
+		oldval: oldval,
+		oldlenp: oldlenp,
+		newval: newval,
+		newlen: newlen
+	};
+
+	return _sysctl(&args);
+}
+#endif
+#endif
 
 //#define __NR_mlock            150
 #ifdef L_mlock
@@ -1277,7 +1422,19 @@ _syscall4(__ptr_t, mremap, __ptr_t, old_address, size_t, old_size, size_t, new_s
 #endif
 
 //#define __NR_setresuid                164
+#ifdef __NR_setresuid
+#ifdef L_setresuid
+_syscall3(int, setresuid, uid_t, ruid, uid_t, euid, uid_t, suid);
+#endif
+#endif
+
 //#define __NR_getresuid                165
+#ifdef __NR_getresuid
+#ifdef L_getresuid
+_syscall3(int, getresuid, uid_t *, euid, uid_t *, ruid, uid_t *, suid);
+#endif
+#endif
+
 //#define __NR_vm86                     166
 
 //#define __NR_query_module             167
@@ -1305,17 +1462,29 @@ _syscall3(int, poll, struct pollfd *, fds, unsigned long int, nfds, int, timeout
 //nfsservctl	EXTRA	nfsservctl	i:ipp	nfsservctl
 
 //#define __NR_setresgid                170
+#ifdef __NR_setresgid
+#ifdef L_setresgid
+_syscall3(int, setresgid, gid_t, rgid, gid_t, egid, gid_t, sgid);
+#endif
+#endif
+
 //#define __NR_getresgid                171
+#ifdef __NR_getresgid
+#ifdef L_getresgid
+_syscall3(int, getresgid, gid_t *, egid, gid_t *, rgid, gid_t *, sgid);
+#endif
+#endif
+
 //#define __NR_prctl                    172
 
 //#define __NR_rt_sigreturn             173
 //#define __NR_rt_sigaction             174
 #ifdef __NR_rt_sigaction
-#define __NR___rt_sigaction __NR_rt_sigaction
-#ifdef L___rt_sigaction
+#define __NR___syscall_rt_sigaction __NR_rt_sigaction
+#ifdef L___syscall_rt_sigaction
 #include <signal.h>
 #undef sigaction
-_syscall4(int, __rt_sigaction, int, signum, const struct sigaction *, act, 
+_syscall4(int, __syscall_rt_sigaction, int, signum, const struct sigaction *, act, 
 		struct sigaction *, oldact, size_t, size); 
 #endif
 #endif
@@ -1352,6 +1521,45 @@ int sigpending(sigset_t *set)
 #endif
 
 //#define __NR_rt_sigtimedwait          177
+#ifdef L___rt_sigtimedwait
+#include <signal.h>
+#define __need_NULL
+#include <stddef.h>
+#ifdef __NR_rt_sigtimedwait
+#define __NR___rt_sigtimedwait __NR_rt_sigtimedwait
+_syscall4(int, __rt_sigtimedwait, const sigset_t *, set, siginfo_t *, info, 
+		const struct timespec *, timeout, size_t, setsize);
+
+int sigwaitinfo(const sigset_t *set, siginfo_t *info)
+{
+	return __rt_sigtimedwait (set, info, NULL, _NSIG/8);
+}
+
+int sigtimedwait (const sigset_t *set, siginfo_t *info, const struct timespec *timeout)
+{
+	return __rt_sigtimedwait (set, info, timeout, _NSIG/8);
+}
+#else
+int sigwaitinfo(const sigset_t *set, siginfo_t *info)
+{
+	if (set==NULL)
+		__set_errno (EINVAL);
+	else
+		__set_errno (ENOSYS);
+	return -1;
+}
+
+int sigtimedwait (const sigset_t *set, siginfo_t *info, const struct timespec *timeout)
+{
+	if (set==NULL)
+		__set_errno (EINVAL);
+	else
+		__set_errno (ENOSYS);
+	return -1;
+}
+#endif
+#endif
+
 //#define __NR_rt_sigqueueinfo          178
 
 //#define __NR_rt_sigsuspend            179
@@ -1370,22 +1578,10 @@ int sigsuspend (const sigset_t *mask)
 #endif
 
 //#define __NR_pread                    180
-#ifdef L___libc_pread
-#define _XOPEN_SOURCE 500
-#include <unistd.h>
-#define __NR___libc_pread __NR_pread
-_syscall4(ssize_t, __libc_pread, int, fd, void *, buf, size_t, count, __off_t, offset);
-weak_alias (__libc_pread, pread)
-#endif
+// See pread_write.c
 
 //#define __NR_pwrite                   181
-#ifdef L___libc_pwrite
-#define _XOPEN_SOURCE 500
-#include <unistd.h>
-#define __NR___libc_pwrite __NR_pwrite
-_syscall4(ssize_t, __libc_pwrite, int, fd, const void *, buf, size_t, count, __off_t, offset);
-weak_alias (__libc_pwrite, pwrite)
-#endif
+// See pread_write.c
 
 //#define __NR_chown                    182
 #ifdef L_chown
@@ -1394,7 +1590,7 @@ _syscall3(int, chown, const char *, path, uid_t, owner, gid_t, group);
 #endif
 
 //#define __NR_getcwd                   183
-// See unistd/getcwd.c -- we don't use this syscall, even when it is available...
+// See getcwd.c in this directory
 
 //#define __NR_capget                   184
 #ifdef L_capget
@@ -1435,7 +1631,7 @@ _syscall2(int, sigaltstack, const struct sigaltstack *, ss, struct sigaltstack *
 #ifdef L_sendfile
 #include <unistd.h>
 #include <sys/sendfile.h>
-_syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, __off_t *, offset, size_t, count)
+_syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, __off_t *, offset, size_t, count);
 #endif
 #endif
 
@@ -1472,84 +1668,69 @@ int getrlimit (__rlimit_resource_t resource, struct rlimit *rlimits)
 
 
 //#define __NR_stat64             195
-#ifdef L___stat64
-#ifdef __UCLIBC_HAVE_LFS__
+#ifdef L___syscall_stat64
+#if defined __NR_stat64 && defined __UCLIBC_HAS_LFS__
+#define __NR___syscall_stat64 __NR_stat64
 #include <unistd.h>
-#include "statfix64.h"
-#define __NR___stat64	__NR_stat64
-extern int __stat64(const char *file_name, struct kernel_stat64 *buf);
-_syscall2(int, __stat64, const char *, file_name, struct kernel_stat64 *, buf);
-
-int __xstat64(int version, const char * file_name, struct libc_stat64 * cstat)
+#include <sys/stat.h>
+#include <bits/kernel_stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_stat64, const char *, file_name, struct kernel_stat64 *, buf);
+int stat64(const char * file_name, struct stat64 * buf)
 {
-	struct kernel_stat64 kstat;
-	int result = __stat64(file_name, &kstat);
-
-	if (result == 0) { 
-		statfix64(cstat, &kstat);
+	int result;
+	struct kernel_stat64 kbuf;
+	result = __syscall_stat64(file_name, &kbuf);
+	if (result == 0) {
+		__xstat64_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int stat64(const char *file_name, struct libc_stat64 *buf)
-{
-	return(__xstat64(0, file_name, buf));
-}
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 #endif
 
 //#define __NR_lstat64            196
-#ifdef L___lstat64
-#ifdef __UCLIBC_HAVE_LFS__
+#ifdef L___syscall_lstat64
+#if defined __NR_lstat64 && defined __UCLIBC_HAS_LFS__
+#define __NR___syscall_lstat64 __NR_lstat64
 #include <unistd.h>
-#include "statfix64.h"
-#define __NR___lstat64	__NR_lstat64
-extern int __lstat64(const char *file_name, struct kernel_stat64 *buf);
-_syscall2(int, __lstat64, const char *, file_name, struct kernel_stat64 *, buf);
-
-int __lxstat64(int version, const char * file_name, struct libc_stat64 * cstat)
+#include <sys/stat.h>
+#include <bits/kernel_stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_lstat64, const char *, file_name, struct kernel_stat64 *, buf);
+int lstat64(const char * file_name, struct stat64 * buf)
 {
-	struct kernel_stat64 kstat;
-	int result = __lstat64(file_name, &kstat);
-
-	if (result == 0) { 
-		statfix64(cstat, &kstat);
+	int result;
+	struct kernel_stat64 kbuf;
+	result = __syscall_lstat64(file_name, &kbuf);
+	if (result == 0) {
+		__xstat64_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int lstat64(const char *file_name, struct libc_stat64 *buf)
-{
-	return(__lxstat64(0, file_name, buf));
-}
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 #endif
 
 //#define __NR_fstat64            197
-#ifdef L___fstat64
-#ifdef __UCLIBC_HAVE_LFS__
+#ifdef L___syscall_fstat64
+#if defined __NR_fstat64 && defined __UCLIBC_HAS_LFS__
+#define __NR___syscall_fstat64 __NR_fstat64
 #include <unistd.h>
-#include "statfix64.h"
-#define __NR___fstat64	__NR_fstat64
-extern int __fstat64(int filedes, struct kernel_stat64 *buf);
-_syscall2(int, __fstat64, int, filedes, struct kernel_stat64 *, buf);
-
-int __fxstat64(int version, int fd, struct libc_stat64 * cstat)
+#include <sys/stat.h>
+#include <bits/kernel_stat.h>
+#include "xstatconv.h"
+_syscall2(int, __syscall_fstat64, int, filedes, struct kernel_stat64 *, buf);
+int fstat64(int fd, struct stat64 * buf)
 {
-	struct kernel_stat64 kstat;
-	int result = __fstat64(fd, &kstat);
-
-	if (result == 0) { 
-		statfix64(cstat, &kstat);
+	int result;
+	struct kernel_stat64 kbuf;
+	result = __syscall_fstat64(fd, &kbuf);
+	if (result == 0) {
+		__xstat64_conv(&kbuf, buf);
 	}
 	return result;
 }
-
-int fstat64(int filedes, struct libc_stat64 *buf)
-{
-	return(__fxstat64(0, filedes, buf));
-}
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 #endif
 
 
@@ -1575,7 +1756,7 @@ int fstat64(int filedes, struct libc_stat64 *buf)
 //#define __NR_pivot_root		217
 #ifdef __NR_pivot_root
 #ifdef L_pivot_root
-_syscall2(int, pivot_root, const char *, new_root, const char *, put_old)
+_syscall2(int, pivot_root, const char *, new_root, const char *, put_old);
 #endif
 #endif
 
@@ -1584,20 +1765,15 @@ _syscall2(int, pivot_root, const char *, new_root, const char *, put_old)
 //#define __NR_madvise1		219	/* delete when C lib stub is removed */
 
 //#define __NR_getdents64		220
-#ifdef L_getdents64
-#ifdef __UCLIBC_HAVE_LFS__
-#include <unistd.h>
-#include <dirent.h>
-_syscall3(int, getdents64, int, fd, char *, dirp, size_t, count);
-#endif /* __UCLIBC_HAVE_LFS__ */
-#endif
+// See getdents64.c
 
 //#define __NR_fcntl64		221
 #ifdef L__fcntl64
-#ifdef __UCLIBC_HAVE_LFS__
-#define __NR__fcntl64 __NR_fcntl64
+#ifdef __UCLIBC_HAS_LFS__
 #include <stdarg.h>
 #include <fcntl.h>
+#ifdef __NR_fcntl64
+#define __NR__fcntl64 __NR_fcntl64
 extern int _fcntl64(int fd, int cmd, long arg);
 
 _syscall3(int, _fcntl64, int, fd, int, cmd, long, arg);
@@ -1613,6 +1789,20 @@ int fcntl64(int fd, int command, ...)
 	va_end(list);
 	return _fcntl64(fd, command, arg);
 }
+#else
+extern int _fcntl(int fd, int cmd, long arg);
+int fcntl64(int fd, int command, ...)
+{
+	long arg;
+	va_list list;
+
+	va_start(list, command);
+	arg = va_arg(list, long);
+
+	va_end(list);
+	return _fcntl(fd, command, arg);
+}
+#endif
 #endif
 #endif
 
